import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.io.*;
import java.util.*;

import GFWindowController;

public class GFCtrl {
    NSWindow window;				// main window
    GFWindowController	controller;    		// controller of setting window
    Vector siteInfo;
    FileAccessor flp;
    NSTextField rFileName; 
    String transferFileName;
    Vector transferFileNames;
    NSTextField progMessage;   
    MyView dropView;
    NSButton sendButton;
    NSPopUpButton connectionMenu;
    NSButtonCell modeAuto;
    NSButtonCell modeAscii;
    NSButtonCell modeBinary;
    NSButtonCell sendAuto;
    NSButtonCell sendSemiAuto;
    NSUserDefaults defaults;
    int transferMode;	// Auto:0, ASCII: 1, BIN 2
    int transferActionMode; // Auto:0, Semi-Auto:1
    int menuItemIndex;  // index of default selected item no of connection Menu

    static final int AUTO = 0;
    static final int ASCII = 1;
    static final int BIN = 2;
    static final int SEMI_AUTO = 1;
    
    public void awakeFromNib(){
//System.out.println("initialization"+System.getProperty("user.dir"));
        defaults = NSUserDefaults.standardUserDefaults();
        menuItemIndex = defaults.integerForKey("connectionMenu");
        transferMode = defaults.integerForKey("transferMode");
        transferActionMode = defaults.integerForKey("transferActionMode");
                
        siteInfo = new Vector();
        transferFileNames = new Vector();
        flp = new FileAccessor();
        flp.readSetting(siteInfo);
        dropView.setBoss(this);
        updateConnectionMenu();
        if(connectionMenu.numberOfItems() > menuItemIndex){
            connectionMenu.selectItemAtIndex(menuItemIndex);
        }
        setTransferModeButton();
        setTransferActionButton();
    }
    private void setTransferModeButton(){
        switch(transferMode){
            case AUTO:
                modeAuto.setIntValue(1);
                modeAscii.setIntValue(0);
                modeBinary.setIntValue(0);
                break;
            case ASCII:
                modeAuto.setIntValue(0);
                modeAscii.setIntValue(1);
                modeBinary.setIntValue(0);
                break;
            case BIN:
                modeAuto.setIntValue(0);
                modeAscii.setIntValue(0);
                modeBinary.setIntValue(1);
                break;
            default:
                break;
        }    
    }
    private void setTransferActionButton(){
         switch(transferActionMode){
            case AUTO:
                setTransferActionModeAuto(null);
                sendAuto.setIntValue(1);
                sendSemiAuto.setIntValue(0);
                break;
            case SEMI_AUTO:
                setTransferActionModeSemiAuto(null);
                sendAuto.setIntValue(0);
                sendSemiAuto.setIntValue(1);
                break;
            default:
                break;
        }   
    }
    private void updateConnectionMenu(){
        connectionMenu.removeAllItems();
        for(int i = 0;i<siteInfo.size();i++){
            ConnectionInfo info = (ConnectionInfo)(siteInfo.elementAt(i));
            connectionMenu.addItem(info.getSetName());
        }
    }
    public void connectionMenuChanged(NSObject sender){
        defaults.setIntegerForKey(connectionMenu.indexOfSelectedItem(),"connectionMenu");   
//System.out.println(" set "+(defaults.integerForKey("connectionMenu")));         
    }
    public void displaySettingWindow(NSObject sender)
    {
        if (controller == null) // Check to see if we've already loaded the second window controller
        {
            controller = new GFWindowController("GFDiag");
            controller.setSiteInfo(siteInfo);
            controller.setBoss(this);
        }
        //Now that we have a controller, tell it to show the window.
        controller.updateInfo();
        controller.showWindow(this);
    }
    
    public void saveSettingAction(){
        flp.fileOutSetting(siteInfo);
        updateConnectionMenu();       
    }

    
    public void sendAction(Object sender) {
       // System.out.println("called");
        //ConnectionInfo info= (ConnectionInfo)(siteInfo.elementAt(connectionMenu.indexOfSelectedItem()));
        //System.out.println(info.getRealRemoteDir(transferFileName));
        //System.out.println(info.getRealFileName(transferFileName));
        dropView.setDraggable(false);//view doesn't wanna accept file anymore        
        startThread();
    }
    
    private void startThread(){
        if(transferFileNames == null) return;
        if(transferFileNames.size() == 0) return;
        //if((transferFileName.trim()).length() == 0) return;
        GFTPControl ftpc = new GFTPControl();
	ftpc.setBoss(this);
        ftpc.setInfo((ConnectionInfo)(siteInfo.elementAt(connectionMenu.indexOfSelectedItem())),
            transferFileNames,transferMode);
    	Thread theThread = new Thread(ftpc);
	theThread.setPriority(Thread.MIN_PRIORITY);
	theThread.start();			
    }    

    ///////////////////////////////////////////////
    ///////////////////////////////////////////////    
    public void setTransferModeAuto(Object sender){
        setTransferMode(AUTO);
    }
    public void setTransferModeASCII(Object sender){
        setTransferMode(ASCII);
    }
    public void setTransferModeBIN(Object sender){
        setTransferMode(BIN);
    }
    private void setTransferMode(int mode){
        transferMode = mode;
        defaults.setIntegerForKey(transferMode,"transferMode");          
    }
    ///////////////////////////////////////////////
    ///////////////////////////////////////////////    
    public void setTransferActionModeAuto(Object sender){
        setTransferActionMode(AUTO);
        sendButton.setTransparent(true);
        sendButton.setEnabled(false);
    }    
    public void setTransferActionModeSemiAuto(Object sender){
        setTransferActionMode(SEMI_AUTO);
        sendButton.setTransparent(false);
        sendButton.setEnabled(true);
    }
    private void setTransferActionMode(int mode){
        transferActionMode = mode;  
//System.out.println(  transferActionMode);      
        defaults.setIntegerForKey(transferActionMode,"transferActionMode");       
//System.out.println(" set "+(defaults.integerForKey("transferActionMode")));             
    }

    ///////////////////////////////////////////////
    /////////Notification from other class    
    ///////////////////////////////////////////////
    public void setFtpStatus(String str){
    // notification from GFTPControl
        progMessage.setStringValue(str);
    }
    // notification from GFTPControl
    
    public void ftpFinished(){
        dropView.setDraggable(true);	// you can work again.
    }
    
    public void fileDropped(NSArray filenames){
        // notification of drag & drop from MyView
        
        transferFileNames.removeAllElements();
        
        ConnectionInfo info= (ConnectionInfo)(siteInfo.elementAt(connectionMenu.indexOfSelectedItem())); 
        java.util.Enumeration enumerator= filenames.objectEnumerator();
        int filecount = 0;
        while(enumerator.hasMoreElements()){
            String filename = (String)(enumerator.nextElement());
            if(info.isValid(filename)){
                String str = info.getShortFileName(filename);
                if(filecount > 0) str = str + " + "+filecount+" files";
                rFileName.setStringValue(str);
                transferFileNames.addElement(filename);
                filecount ++;
            }
            else{
                rFileName.setStringValue("Error:File is not in specified Folder");
            }    
        }
        if(transferActionMode==AUTO){
            sendAction(null);
        }
    }    
    
    ///////////////////////////////////////////////
    ///////////////////////////////////////////////    
    public  void applicationDidFinishLaunching(NSNotification aNotification){
       /* System.out.println("awakening");
        defaults = NSUserDefaults.standardUserDefaults();
        menuItemIndex = defaults.integerForKey("connectionMenu");
        transferMode = defaults.integerForKey("transferMode");
        transferActionMode = defaults.integerForKey("transferActionMode");      */
    }
    
    public void applicationWillTerminate(NSNotification aNotification){
        defaults.synchronize();
    }
}


