/*
 MyView.java
 JDragNDrop
*/

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

public class MyView extends NSView {
    
    // these are the Interface elements to feed information to...
    private NSTextField rFileName;
    private GFCtrl boss;
    private boolean draggableFlag;
    
    // override constructor to add dragging registration
    public MyView(NSRect frame) {
        super(frame);
        
        // register for drag and drop type - filenames...
        NSMutableArray dragTypesArray = new NSMutableArray();
        dragTypesArray.addObject(NSPasteboard.FilenamesPboardType);
        
        registerForDraggedTypes((NSArray) dragTypesArray);
        draggableFlag = true;
    }
    public void setBoss(GFCtrl _boss){
        boss = _boss;
    }
    public void setDraggable(boolean flag){
        draggableFlag = flag;
    }
    // draw background and fill in color
    public void drawRect(NSRect rect) {
        NSColor myColor = new NSColor();
        NSBezierPath myBezierPath = new NSBezierPath();
        
        myColor.windowBackgroundColor().set();
        myBezierPath.fillRect(this.bounds());
    }
    
    public String validateDrag(NSDraggingInfo sender) {
        // if the source of the drag operation does not originate from this view...
        if(sender.draggingSource() != this) {
            // get the pasteboard and get the type of item dropped
            NSPasteboard pb = sender.draggingPasteboard();
            String type = pb.availableTypeFromArray(new NSArray(NSPasteboard.FilenamesPboardType));
            
            // if type exists return the string for it else return null
            if(type != null) {
                return pb.stringForType(type);
            }
        }
        return null;
    }

    public int draggingEntered(NSDraggingInfo sender) {
        // if drag operation is valid, let the pasteboard know
        // the app is only copying the type (filename in this case)
        if((validateDrag(sender) != null)&& draggableFlag) {
            return NSDraggingInfo.DragOperationCopy;
        }
        // else return no operation
        return NSDraggingInfo.DragOperationNone;
    }
    
    public int draggingUpdated(NSDraggingInfo sender) {
        // if drag operation is valid, let the pasteboard know
        // the app is only copying the type (filename in this case)
        if((validateDrag(sender) != null)&&draggableFlag) {
            return NSDraggingInfo.DragOperationCopy;
        }
        // else return no operation
        return NSDraggingInfo.DragOperationNone;
    }

    public void draggingExited(NSDraggingInfo sender) {
        // once drag operation exits window, set display to view again...
        this.setNeedsDisplay(true);
    }
    
    public boolean prepareForDragOperation(NSDraggingInfo sender) {
        // if dragging is valid, let the pasteboard know the app is prepared
        if((validateDrag(sender) != null)&&draggableFlag) {
            return true;
        }
        
        return false;
    }
    
    public boolean performDragOperation(NSDraggingInfo sender) {
        // if dragging is valid, let the pasteboard know the app is performing operation
        if((validateDrag(sender) != null)&&draggableFlag){
            return true;
        }
        return false;
    }
    
    public void concludeDragOperation(NSDraggingInfo sender) {

        // create a new NSPasteboard object to confirm type of dropped item
        NSPasteboard pboard = sender.draggingPasteboard();
        NSMutableArray pArray = new NSMutableArray();

        pArray.addObject(NSPasteboard.FilenamesPboardType);

        // check type to make sure it conforms with type you registered
        String type = pboard.availableTypeFromArray((NSArray) pArray);
        
        // if type of item is dropped is File/Folder
        if (type.equalsIgnoreCase(NSPasteboard.FilenamesPboardType)) {
        
            // get the list of all files/folders dropped
            NSArray filenames = (NSArray) pboard.propertyListForType(NSPasteboard.FilenamesPboardType);
            
            // NSFileWrapper is useful for getting the icon of the dropped item
            NSFileWrapper droppedFile = new NSFileWrapper((String)filenames.lastObject(), false);

            // this will set filename to the full path of the dropped item
            String filename = (String)filenames.objectAtIndex(0);
          //  rFileName.setStringValue(filename);
            
            // this will set the icon to the icon of the file/folder dropped
           // iconwell.setImage(droppedFile.icon());
           
           boss.fileDropped(filenames);

        }
        else {
            NSAlertPanel alert = new NSAlertPanel();
            alert.runAlert("Incorrect Type", "The view has not registered for this drag type", null, null, null);
        }
    }
}
